; Fastboot: creates the file /etc/fastboot
; then calls a program that is pointed to by the string SDMASTER.
;
; Purpose: make a MiNT fastboot (i.e. without the initial filesystem check)
; using the AES shutdown master while the AES is open on the console.
;
; (c) 1997 by Draco

	include	aesconst.s
	include	gemdos.i
	include	gemmacro.i

; The stack must be large enough, otherwise we will get bus errors.

STACK		equ	$0400
		
; Standard initialization.

	SECTION	TEXT

real.start
	move.l	$04(sp),a5	;basepage pointer to a5
	move.l	$18(a5),a6	;base of BSS segment
	add.l	$1c(a5),a6	;size of BSS segment	
	lea	STACK(a6),sp	;new stack to sp
	move.l	$0c(a5),d0	;size of text segment
	add.l	$14(a5),d0	;size of data segment
	add.l	$1c(a5),d0	;base of BSS segment
	add.l	#STACK+$0100,d0	;size of stack
	move.l	d0,-(sp)
	move.l	a5,-(sp)
	clr.w	-(sp)
	move.w	#M_shrink,-(sp)
	trap	#1
	lea	$0c(sp),sp

; We must get ap_id because we will act using AES.

	appl_init
	move.w	d0,ap_id
	bmi.w	exit

; We won't draw anything, but we must initialize the damned virtual
; workstation, otherwise this process will be replaced with the
; application launched with shel_write()!

	graf_handle
	lea	intin,a0
	moveq	#$09,d0
fill	move.w	#$01,(a0)+
	dbra	d0,fill
	move.w	#$02,(a0)
	v_opnvwk

; Standard AES 4.10 stuff here...

	menu_register	ap_id,#fname
	shel_write	#SWM_NEWMSG,#NM_APTERM,#NULL,#NULL,#NULL

; Now we must check, if the caller has root privileges.
; GEM is actually always a root, but better to be safe ;)

	move.w	#P_geteuid,-(sp)
	trap	#1
	addq.l	#$02,sp
	tst.w	d0
	bne	noroot

; Now we're looking who's an SDMASTER here... The system documentation
; doesn't say anyshit on what occurs if there's no such a string in the
; system environment, but we can assume, that the returned pointer is NULL then.
; N.B. We're in assembler, so we can safely `make integer from pointer without a cast'
; as we are sure that the compiler won't complain about (unlike the damned C).

	shel_envrn	#spath,#intxt
	tst.l	spath
	beq.w	nosd

; SDMASTER seems to exist, now we will try to create fastboot. Considering
; the general purpose of this program, we could easily assume we'll act
; on the Minix filesystem disk. This assumption is, however, useless, because,
; once the file is created, no user could delete it regardless of access
; attributes. That is because we're root and the destination directory is ours. ;)

	clr.w	-(sp)
	pea	fpath
	move.w	#F_create,-(sp)
	trap	#1
	addq.l	#$08,sp
	move.w	d0,fstat
	bmi	.sk

	move.w	d0,-(sp)
	move.w	#F_close,-(sp)
	trap	#1
	addq.l	#$04,sp

; Now we place /etc/nologin to prevent new users from logging on after
; we initiated the shutdown process.

.sk	clr.w	-(sp)
	pea	nolog
	move.w	#F_create,-(sp)
	trap	#1
	addq.l	#$08,sp
	tst.w	d0
	bmi	nounix
	tst.w	fstat
	bmi	nounix

	move.w	fstat,-(sp)
	move.w	#F_close,-(sp)
	trap	#1
	addq.l	#$04,sp

; Everything went OK, so we call the local Shutdown Master now.

go	shel_write	#SWM_LAUNCH,#NULL,#NULL,spath,#NULL
	move.w	int_out,d0
	beq	term

; Now we're waiting for messages we could be eventually interested in:
; - CH_EXIT: child program exited, we assume that it was cancelled.
; - AP_TERM: the exspected shutdown process has been taken into effect.
; We should immediately quit on AP_TERM, not giving the system a chance
; to use SIGKILL against us. ;)

wait	evnt_mesag	#aesbuf
	move.w	aesbuf,d0
	cmpi.w	#CH_EXIT,d0
	beq.s	term
	cmpi.w	#AP_TERM,d0
	bne.s	wait
	bra	exit		

; If the shutdown was cancelled, we must delete files we have placed before.

term	pea	fpath
	move.w	#F_delete,-(sp)
	trap	#1
	addq.l	#$06,sp
	pea	nolog
	move.w	#F_delete,-(sp)
	trap	#1
	addq.l	#$06,sp
	bra.s	exit

noroot	form_alert	#1,#nosuper
	bra.s	exit
nounix	move.w	d0,fstat
	form_alert	#1,#nominix
	cmpi.w	#$01,int_out
	beq	go
	bra.s	exit
nosd	form_alert	#1,#nomaste
exit	v_clsvwk
	appl_exit
	clr.w	-(sp)
	trap	#1

	include	aeslib.s
	include	vdilib.s

	SECTION DATA

intxt	dc.b	"SDMASTER=",$00
nolog	dc.b	"u:\etc\nologin",$00
fpath	dc.b	"u:\etc\fastboot",$00
fname	dc.b	"  MiNT Fastboot",$00

nosuper	dc.b	"[4][|We have to be root|to shutdown the system, Sir.][ Ahh! ]",$00
nomaste dc.b	"[4][|There's no Shutdown Master|defined for this system, Sir.][ OK ]",$00
nominix	dc.b	"[4][|It seems like we have missing /etc, Sir.|Shall we continue?][ Yes ][ No ]",$00

	SECTION BSS

aesbuf	ds.w	8
fstat	ds.w	1
spath	ds.l	1
ap_id	ds.w	1
